@@ -111,13 +111,21 @@ public:
     int buildPattern(StrVector &sequences, char *sequence_type, int nseq, int nsite);
 
     /**
-            read the alignment in PHYLIP format
+            read the alignment in PHYLIP format (interleaved)
             @param filename file name
             @param sequence_type type of the sequence, either "BIN", "DNA", "AA", or NULL
             @return 1 on success, 0 on failure
      */
     int readPhylip(char *filename, char *sequence_type);
 
+    /**
+            read the alignment in sequential PHYLIP format
+            @param filename file name
+            @param sequence_type type of the sequence, either "BIN", "DNA", "AA", or NULL
+            @return 1 on success, 0 on failure
+     */
+    int readPhylipSequential(char *filename, char *sequence_type);
+
     /**
             read the alignment in FASTA format
             @param filename file name
@@ -223,7 +231,7 @@ public:
     		bool exclude_gaps = false, bool exclude_const_sites = false, const char *ref_seq_name = NULL);
 
     void printPhylip(ostream &out, bool append = false, const char *aln_site_list = NULL,
-    		bool exclude_gaps = false, bool exclude_const_sites = false, const char *ref_seq_name = NULL);
+    		bool exclude_gaps = false, bool exclude_const_sites = false, const char *ref_seq_name = NULL, bool print_taxid = false);
 
     void printFasta(const char *filename, bool append = false, const char *aln_site_list = NULL,
     		bool exclude_gaps = false, bool exclude_const_sites = false, const char *ref_seq_name = NULL);
@@ -352,8 +360,10 @@ public:
             @param aln original input alignment
             @param seq_id ID of sequences to extract from
             @param min_true_cher the minimum number of non-gap characters, true_char<min_true_char -> delete the sequence
+            @param min_taxa only keep alignment that has >= min_taxa sequences
+            @param[out] kept_partitions (for SuperAlignment) indices of kept partitions
      */
-    virtual void extractSubAlignment(Alignment *aln, IntVector &seq_id, int min_true_char);
+    virtual void extractSubAlignment(Alignment *aln, IntVector &seq_id, int min_true_char, int min_taxa = 0, IntVector *kept_partitions = NULL);
 
     /**
             extract a sub-set of patterns
@@ -391,8 +401,9 @@ public:
             resampling pattern frequency by a non-parametric bootstrap
             @param pattern_freq (OUT) resampled pattern frequencies
             @param spec bootstrap specification, see above
+            @param rstream random generator stream, NULL to use the global randstream
      */
-    virtual void createBootstrapAlignment(int *pattern_freq, const char *spec = NULL);
+    virtual void createBootstrapAlignment(int *pattern_freq, const char *spec = NULL, int *rstream = NULL);
 
     /**
             create a gap masked alignment from an input alignment. Gap patterns of masked_aln 
@@ -539,13 +550,13 @@ public:
             compute empirical rates between state pairs
             @param rates (OUT) vector of size num_states*(num_states-1)/2 for the rates
      */
-    virtual void computeEmpiricalRate(double *rates);
+    virtual void computeDivergenceMatrix(double *rates);
 
     /**
             compute non-reversible empirical rates between state pairs
             @param rates (OUT) vector of size num_states*(num_states-1) for the rates
      */
-    virtual void computeEmpiricalRateNonRev(double *rates);
+    virtual void computeDivergenceMatrixNonRev(double *rates);
 
     /**
             count the fraction of constant sites in the alignment, update the variable frac_const_sites
