@@ -54,6 +54,30 @@ RateFree::RateFree(int ncat, double start_alpha, string params, bool sorted_rate
 	}
 }
 
+void RateFree::saveCheckpoint() {
+    checkpoint->startStruct("RateFree");
+//    CKP_SAVE(fix_params);
+//    CKP_SAVE(sorted_rates);
+//    CKP_SAVE(optimize_alg);
+    CKP_ARRAY_SAVE(ncategory, prop);
+    CKP_ARRAY_SAVE(ncategory, rates);
+    checkpoint->endStruct();
+    RateGamma::saveCheckpoint();
+}
+
+void RateFree::restoreCheckpoint() {
+    RateGamma::restoreCheckpoint();
+    checkpoint->startStruct("RateFree");
+//    CKP_RESTORE(fix_params);
+//    CKP_RESTORE(sorted_rates);
+//    CKP_RESTORE(optimize_alg);
+    CKP_ARRAY_RESTORE(ncategory, prop);
+    CKP_ARRAY_RESTORE(ncategory, rates);
+    checkpoint->endStruct();
+
+//	setNCategory(ncategory);
+}
+
 void RateFree::setNCategory(int ncat) {
 
     // initialize with gamma rates
@@ -63,7 +87,7 @@ void RateFree::setNCategory(int ncat) {
 
     int i;
 	for (i = 0; i < ncategory; i++)
-        prop[i] = 1.0/ncategory;
+        prop[i] = (1.0-getPInvar())/ncategory;
     
 //	double sum_prop = (ncategory)*(ncategory+1)/2.0;
 //	double sum = 0.0;
@@ -86,6 +110,7 @@ void RateFree::setNCategory(int ncat) {
 
 void RateFree::setRateAndProp(RateFree *input) {
     assert(input->ncategory == ncategory-1);
+    setPInvar(input->getPInvar());
     int k = 0, i;
     // get the category k with largest proportion
     for (i = 1; i < ncategory-1; i++)
@@ -224,6 +249,7 @@ double RateFree::optimizeParameters(double gradient_epsilon) {
         if (sorted_rates)
             quicksort(rates, 0, ncategory-1, prop);
         phylo_tree->clearAllPartialLH();
+        score = phylo_tree->computeLikelihood();
     }
     optimizing_params = 0;
 
@@ -247,7 +273,7 @@ void RateFree::setBounds(double *lower_bound, double *upper_bound, bool *bound_c
         }
     } else if (optimizing_params == 1){
         // rates
-        for (i = 1; i <= ncategory; i++) {
+        for (i = 1; i < ncategory; i++) {
             lower_bound[i] = MIN_FREE_RATE;
             upper_bound[i] = MAX_FREE_RATE;
             bound_check[i] = false;
@@ -426,6 +452,7 @@ double RateFree::optimizeWithEM() {
     size_t ptn, c;
     size_t nptn = phylo_tree->aln->getNPattern();
     size_t nmix = ncategory;
+    const double MIN_PROP = 1e-4;
     
 //    double *lk_ptn = aligned_alloc<double>(nptn);
     double *new_prop = aligned_alloc<double>(nmix);
@@ -445,22 +472,34 @@ double RateFree::optimizeWithEM() {
     model_fac->site_rate = site_rate;
     tree->model_factory = model_fac;
     tree->setParams(phylo_tree->params);
-        
+    double old_score = 0.0;
     // EM algorithm loop described in Wang, Li, Susko, and Roger (2008)
     for (int step = 0; step < ncategory; step++) {
         // first compute _pattern_lh_cat
         double score;
         score = phylo_tree->computePatternLhCat(WSL_RATECAT);
+        if (score > 0.0) {
+            phylo_tree->printTree(cout, WT_BR_LEN+WT_NEWLINE);
+            writeInfo(cout);
+        }
+        assert(score < 0);
+        
+        if (step > 0)
+            assert(score > old_score-0.1);
+            
+        old_score = score;
+        
         memset(new_prop, 0, nmix*sizeof(double));
                 
         // E-step
         // decoupled weights (prop) from _pattern_lh_cat to obtain L_ci and compute pattern likelihood L_i
         for (ptn = 0; ptn < nptn; ptn++) {
             double *this_lk_cat = phylo_tree->_pattern_lh_cat + ptn*nmix;
-            double lk_ptn = 0.0;
+            double lk_ptn = phylo_tree->ptn_invar[ptn];
             for (c = 0; c < nmix; c++) {
                 lk_ptn += this_lk_cat[c];
             }
+            assert(lk_ptn != 0.0);
             lk_ptn = phylo_tree->ptn_freq[ptn] / lk_ptn;
             
             // transform _pattern_lh_cat into posterior probabilities of each category
@@ -472,14 +511,46 @@ double RateFree::optimizeWithEM() {
         } 
         
         // M-step, update weights according to (*)        
-        
-        bool converged = true;
+        int maxpropid = 0;
+        double new_pinvar = 0.0;    
         for (c = 0; c < nmix; c++) {
             new_prop[c] = new_prop[c] / phylo_tree->getAlnNSite();
+            if (new_prop[c] > new_prop[maxpropid])
+                maxpropid = c;
+        }
+        // regularize prop
+        bool zero_prop = false;
+        for (c = 0; c < nmix; c++) {
+            if (new_prop[c] < MIN_PROP) {
+                new_prop[maxpropid] -= (MIN_PROP - new_prop[c]);
+                new_prop[c] = MIN_PROP;
+                zero_prop = true;
+            }
+        }
+        // break if some probabilities too small
+        if (zero_prop) break;
+        
+        bool converged = true;
+        double sum_prop = 0.0;
+        for (c = 0; c < nmix; c++) {
+//            new_prop[c] = new_prop[c] / phylo_tree->getAlnNSite();
             // check for convergence
+            sum_prop += new_prop[c];
             converged = converged && (fabs(prop[c]-new_prop[c]) < 1e-4);
             prop[c] = new_prop[c];
+            new_pinvar += new_prop[c];
         }
+
+        new_pinvar = 1.0 - new_pinvar;
+
+        if (new_pinvar != 0.0) {
+            converged = converged && (fabs(getPInvar()-new_pinvar) < 1e-4);
+            setPInvar(new_pinvar);
+//            setOptimizePInvar(false);
+            phylo_tree->computePtnInvar();
+        }
+        
+        assert(fabs(sum_prop+new_pinvar-1.0) < MIN_PROP);
         
         // now optimize rates one by one
         double sum = 0.0;
@@ -506,7 +577,7 @@ double RateFree::optimizeWithEM() {
                 tree->ptn_freq[ptn] = this_lk_cat[ptn*nmix];
             double scaling = rates[c];
             tree->scaleLength(scaling);
-            tree->optimizeTreeLengthScaling(scaling, 0.001);
+            tree->optimizeTreeLengthScaling(MIN_PROP, scaling, 1.0/prop[c], 0.001);
             converged = converged && (fabs(rates[c] - scaling) < 1e-4);
             rates[c] = scaling;
             sum += prop[c] * rates[c];
