@@ -154,7 +154,7 @@ public:
     void printTree(ostream & out, int brtype = WT_BR_LEN);
 
 
-    string getTreeString();
+//    string getTreeString();
 
     /**
             print the tree to the output file in newick format
