@@ -69,6 +69,13 @@ Numeric PhyloTree::dotProductSIMD(Numeric *x, Numeric *y, int size) {
 
 template <class VectorClass, const int VCSIZE, const int nstates>
 void PhyloTree::computePartialLikelihoodEigenSIMD(PhyloNeighbor *dad_branch, PhyloNode *dad) {
+
+    if (dad_branch->node->degree() > 3) {
+        // TODO: SIMD version for multifurcating node
+        computePartialLikelihoodEigen(dad_branch, dad);
+        return;
+    }
+
     // don't recompute the likelihood
 	assert(dad);
     if (dad_branch->partial_lh_computed & 1)
@@ -80,12 +87,12 @@ void PhyloTree::computePartialLikelihoodEigenSIMD(PhyloNeighbor *dad_branch, Phy
     size_t nptn = aln->size() + model_factory->unobserved_ptns.size();
     PhyloNode *node = (PhyloNode*)(dad_branch->node);
 
+    if (!tip_partial_lh_computed)
+        computeTipPartialLikelihood();
+
 	if (node->isLeaf()) {
 	    dad_branch->lh_scale_factor = 0.0;
 	    //memset(dad_branch->scale_num, 0, nptn * sizeof(UBYTE));
-
-		if (!tip_partial_lh_computed)
-			computeTipPartialLikelihood();
 		return;
 	}
 
@@ -371,7 +378,7 @@ void PhyloTree::computePartialLikelihoodEigenSIMD(PhyloNeighbor *dad_branch, Phy
 			}
             // check if one should scale partial likelihoods
 			double lh_max = horizontal_max(vc_max);
-            if (lh_max < SCALING_THRESHOLD) {
+            if (lh_max < SCALING_THRESHOLD && ptn_invar[ptn] == 0.0) {
             	// now do the likelihood scaling
             	partial_lh -= block; // revert its pointer
             	VectorClass scale_thres(SCALING_THRESHOLD_INVER);
@@ -453,7 +460,7 @@ void PhyloTree::computePartialLikelihoodEigenSIMD(PhyloNeighbor *dad_branch, Phy
 
             // check if one should scale partial likelihoods
 			double lh_max = horizontal_max(vc_max);
-            if (lh_max < SCALING_THRESHOLD) {
+            if (lh_max < SCALING_THRESHOLD && ptn_invar[ptn] == 0.0) {
 				// now do the likelihood scaling
             	partial_lh -= block; // revert its pointer
             	VectorClass scale_thres(SCALING_THRESHOLD_INVER);
@@ -883,6 +890,12 @@ double PhyloTree::computeLikelihoodBranchEigenSIMD(PhyloNeighbor *dad_branch, Ph
 						vc_ptn[j] = mul_add(vc_val[i] * vc_tip_partial_lh[j*(nstates/VCSIZE)+i%(nstates/VCSIZE)],
 								vc_partial_lh_dad[j], vc_ptn[j]);
 					}
+                    
+                // bugfix 2016-01-21, prob_const can be rescaled
+                for (j = 0; j < VCSIZE; j++)
+                    if (dad_branch->scale_num[ptn+j] >= 1)
+                        vc_ptn[j] = vc_ptn[j] * SCALING_THRESHOLD;
+
 				// ptn_invar[ptn] is not aligned
 				lh_ptn = horizontal_add(vc_ptn) + VectorClass().load(&ptn_invar[ptn]);
 			}
@@ -973,6 +986,11 @@ double PhyloTree::computeLikelihoodBranchEigenSIMD(PhyloNeighbor *dad_branch, Ph
 					}
 				}
 
+                // bugfix 2016-01-21, prob_const can be rescaled
+                for (j = 0; j < VCSIZE; j++)
+                    if (dad_branch->scale_num[ptn+j] + node_branch->scale_num[ptn+j] >= 1)
+                        vc_ptn[j] = vc_ptn[j] * SCALING_THRESHOLD;
+
 				// ptn_invar[ptn] is not aligned
 				lh_ptn = horizontal_add(vc_ptn) + VectorClass().load(&ptn_invar[ptn]);
 				partial_lh_node += block*VCSIZE;
@@ -990,6 +1008,7 @@ double PhyloTree::computeLikelihoodBranchEigenSIMD(PhyloNeighbor *dad_branch, Ph
 
 	if (orig_nptn < nptn) {
     	// ascertainment bias correction
+        assert(prob_const < 1.0 && prob_const >= 0.0);
     	prob_const = log(1.0 - prob_const);
     	for (ptn = 0; ptn < orig_nptn; ptn++)
     		_pattern_lh[ptn] -= prob_const;
@@ -1108,7 +1127,19 @@ double PhyloTree::computeLikelihoodFromBufferEigenSIMD() {
 		lh_ptn = 0.0;
 		double prob_const;// df_const, ddf_const;
 		double *theta = &theta_all[orig_nptn*block];
-		for (ptn = orig_nptn; ptn < nptn; ptn+=VCSIZE) {
+
+        UBYTE sum_scale_num[nstates+VCSIZE];
+        memset(sum_scale_num, 0, sizeof(UBYTE)*(nstates+VCSIZE));
+        if (current_it->node->isLeaf())
+            memcpy(sum_scale_num, current_it_back->scale_num+orig_nptn, sizeof(UBYTE)*(nptn-orig_nptn));
+        else if (current_it_back->node->isLeaf())
+            memcpy(sum_scale_num, current_it->scale_num+orig_nptn, sizeof(UBYTE)*(nptn-orig_nptn));
+        else {
+            for (ptn = orig_nptn; ptn < nptn; ptn++)
+                sum_scale_num[ptn-orig_nptn] = current_it->scale_num[ptn] + current_it_back->scale_num[ptn];
+        }
+
+        for (ptn = orig_nptn; ptn < nptn; ptn+=VCSIZE) {
 			lh_final += lh_ptn;
 
 			// initialization
@@ -1123,6 +1154,11 @@ double PhyloTree::computeLikelihoodFromBufferEigenSIMD() {
 			}
 			theta += block*VCSIZE;
 
+            // bugfix 2016-01-21, prob_const can be rescaled
+            for (j = 0; j < VCSIZE; j++)
+                if (sum_scale_num[ptn+j-orig_nptn] >= 1)
+                    vc_ptn[j] = vc_ptn[j] * SCALING_THRESHOLD;
+
 			// ptn_invar[ptn] is not aligned
 			lh_ptn = horizontal_add(vc_ptn) + VectorClass().load(&ptn_invar[ptn]);
 
