@@ -21,6 +21,12 @@
 
 template <class VectorClass, const int VCSIZE, const int nstates>
 void PhyloTree::computeMixratePartialLikelihoodEigenSIMD(PhyloNeighbor *dad_branch, PhyloNode *dad) {
+    if (dad_branch->node->degree() > 3) {
+        // TODO: SIMD version for multifurcating node
+        computeMixratePartialLikelihoodEigen(dad_branch, dad);
+        return;
+    }
+
     // don't recompute the likelihood
 	assert(dad);
     if (dad_branch->partial_lh_computed & 1)
@@ -838,6 +844,10 @@ double PhyloTree::computeMixrateLikelihoodBranchEigenSIMD(PhyloNeighbor *dad_bra
 								VectorClass().load_a(&partial_lh_dad[i]), vc_ptn[j]);
 					}
 				}
+                // bugfix 2016-01-21, prob_const can be rescaled
+                for (j = 0; j < VCSIZE; j++)
+                    if (dad_branch->scale_num[ptn+j] >= 1)
+                        vc_ptn[j] = vc_ptn[j] * SCALING_THRESHOLD;
 				// ptn_invar[ptn] is not aligned
 				lh_ptn = horizontal_add(vc_ptn) + VectorClass().load(&ptn_invar[ptn]);
 			}
@@ -930,6 +940,11 @@ double PhyloTree::computeMixrateLikelihoodBranchEigenSIMD(PhyloNeighbor *dad_bra
 					}
 				}
 
+                // bugfix 2016-01-21, prob_const can be rescaled
+                for (j = 0; j < VCSIZE; j++)
+                    if (dad_branch->scale_num[ptn+j] + node_branch->scale_num[ptn+j] >= 1)
+                        vc_ptn[j] = vc_ptn[j] * SCALING_THRESHOLD;
+
 				// ptn_invar[ptn] is not aligned
 				lh_ptn = horizontal_add(vc_ptn) + VectorClass().load(&ptn_invar[ptn]);
 				partial_lh_node += block*VCSIZE;
@@ -1065,6 +1080,18 @@ double PhyloTree::computeMixrateLikelihoodFromBufferEigenSIMD() {
 		lh_ptn = 0.0;
 		double prob_const;// df_const, ddf_const;
 		double *theta = &theta_all[orig_nptn*block];
+
+        UBYTE sum_scale_num[nstates+VCSIZE];
+        memset(sum_scale_num, 0, sizeof(UBYTE)*(nstates+VCSIZE));
+        if (current_it->node->isLeaf())
+            memcpy(sum_scale_num, current_it_back->scale_num+orig_nptn, sizeof(UBYTE)*(nptn-orig_nptn));
+        else if (current_it_back->node->isLeaf())
+            memcpy(sum_scale_num, current_it->scale_num+orig_nptn, sizeof(UBYTE)*(nptn-orig_nptn));
+        else {
+            for (ptn = orig_nptn; ptn < nptn; ptn++)
+                sum_scale_num[ptn-orig_nptn] = current_it->scale_num[ptn] + current_it_back->scale_num[ptn];
+        }
+
 		for (ptn = orig_nptn; ptn < nptn; ptn+=VCSIZE) {
 			lh_final += lh_ptn;
 
@@ -1080,6 +1107,11 @@ double PhyloTree::computeMixrateLikelihoodFromBufferEigenSIMD() {
 			}
 			theta += block*VCSIZE;
 
+            // bugfix 2016-01-21, prob_const can be rescaled
+            for (j = 0; j < VCSIZE; j++)
+                if (sum_scale_num[ptn+j-orig_nptn] >= 1)
+                    vc_ptn[j] = vc_ptn[j] * SCALING_THRESHOLD;
+
 			// ptn_invar[ptn] is not aligned
 			lh_ptn = horizontal_add(vc_ptn) + VectorClass().load(&ptn_invar[ptn]);
 
