@@ -24,6 +24,7 @@
 #include "model/rateinvar.h"
 #include "model/rategammainvar.h"
 #include "model/ratefree.h"
+#include "model/ratefreeinvar.h"
 //#include "modeltest_wrapper.h"
 #include "model/modelprotein.h"
 #include "model/modelbin.h"
@@ -33,6 +34,8 @@
 #include "timeutil.h"
 
 #include "phyloanalysis.h"
+#include "gsl/mygsl.h"
+#include "vectorclass/vectorclass.h"
 
 
 /******* Binary model set ******/
@@ -269,28 +272,28 @@ void printSiteLhCategory(const char*filename, PhyloTree *tree, SiteLoglType wsl)
 		out.open(filename);
 		out << "Note : P(D|M) is the probability of site D given the model M (i.e., the site likelihood)" << endl;
         if (wsl == WSL_RATECAT) {
-            out << "P(D|M,rr[x]) is the probability of site D given the model M and the relative rate" << endl;
-            out << "of evolution rr[x], where x is the class of rate to be considered." << endl;
-            out << "We have P(D|M) = \\sum_x P(x) x P(D|M,rr[x])." << endl << endl;
+            out << "P(D|M,rr[i]) is the probability of site D given the model M and the relative rate" << endl;
+            out << "of evolution rr[i], where i is the class of rate to be considered." << endl;
+            out << "We have P(D|M) = \\sum_i P(i) x P(D|M,rr[i])." << endl << endl;
             out << "Site   logP(D|M)       ";
             for (i = 0; i < ncat; i++)
-                out << "logP(D|M,rr[" << i+1 << "]=" << tree->getRate()->getRate(i)<< ") ";
+                out << "log{P(" << i+1 << ")xP(D|M,rr[" << i+1 << "]=" << tree->getRate()->getRate(i)<< ")} ";
         } else if (wsl == WSL_MIXTURE) {
-            out << "P(D|M[x]) is the probability of site D given the model M[x]," << endl;
-            out << "where x is the mixture class to be considered." << endl;
-            out << "We have P(D|M) = \\sum_x P(x) x P(D|M[x])." << endl << endl;
+            out << "P(D|M[i]) is the probability of site D given the model M[i]," << endl;
+            out << "where i is the mixture class to be considered." << endl;
+            out << "We have P(D|M) = \\sum_i P(i) x P(D|M[i])." << endl << endl;
             out << "Site   logP(D|M)       ";
             for (i = 0; i < ncat; i++)
-                out << "logP(D|M[" << i+1 << "]) ";
+                out << "log{P(" << i+1 << ")xP(D|M[" << i+1 << "])} ";
         } else {
             // WSL_MIXTURE_RATECAT
-            out << "P(D|M[x],rr[y]) is the probability of site D given the model M[x] and the relative rate" << endl;
-            out << "of evolution rr[y], where x and y are the mixture class and rate class, respectively." << endl;
-            out << "We have P(D|M) = \\sum_x \\sum_y P(x) x P(y) x P(D|M[x],rr[y])." << endl << endl;
+            out << "P(D|M[i],rr[j]) is the probability of site D given the model M[i] and the relative rate" << endl;
+            out << "of evolution rr[j], where i and j are the mixture class and rate class, respectively." << endl;
+            out << "We have P(D|M) = \\sum_i \\sum_j P(i) x P(j) x P(D|M[i],rr[j])." << endl << endl;
             out << "Site   logP(D|M)       ";
             for (i = 0; i < tree->getModel()->getNMixtures(); i++)
                 for (int j = 0; j < tree->getRate()->getNRate(); j++) {
-                    out << "logP(D|M[" << i+1 << "],rr[" << j+1 << "]=" << tree->getRate()->getRate(j) << ") ";
+                    out << "log{P(" << i+1 << ")xP(" << j+1 << ")xP(D|M[" << i+1 << "],rr[" << j+1 << "]=" << tree->getRate()->getRate(j) << ")} ";
                 }
         }
 		out << endl;
@@ -331,6 +334,37 @@ void printSiteLhCategory(const char*filename, PhyloTree *tree, SiteLoglType wsl)
 
 }
 
+void printSiteStateFreq(const char*filename, PhyloTree *tree) {
+
+    int i, j, nsites = tree->getAlnNSite(), nstates = tree->aln->num_states;
+    double *ptn_state_freq = new double[tree->getAlnNPattern() * nstates];
+    
+    tree->computePatternStateFreq(ptn_state_freq);
+
+	try {
+		ofstream out;
+		out.exceptions(ios::failbit | ios::badbit);
+		out.open(filename);
+		IntVector pattern_index;
+		tree->aln->getSitePatternIndex(pattern_index);
+		for (i = 0; i < nsites; i++) {
+			out.width(6);
+			out << left << i+1 << " ";
+            double *state_freq = &ptn_state_freq[pattern_index[i]*nstates];
+			for (j = 0; j < nstates; j++) {
+				out.width(15);
+				out << state_freq[j] << " ";
+			}
+			out << endl;
+		}
+		out.close();
+		cout << "Site state frequency vectors printed to " << filename << endl;
+	} catch (ios::failure) {
+		outError(ERR_WRITE_OUTPUT, filename);
+	}
+    delete [] ptn_state_freq;
+}
+
 bool checkModelFile(ifstream &in, bool is_partitioned, vector<ModelInfo> &infos) {
 	if (!in.is_open()) return false;
 	in.exceptions(ios::badbit);
@@ -526,7 +560,8 @@ int getModelList(Params &params, Alignment *aln, StrVector &models, bool separat
                     // disallow MG+F
                     if (freq_names[i] == "+F" && orig_model_names[j].find("MG") != string::npos)
                         continue;
-                    if (freq_names[i] != "" || model_type == 2) // empirical model also allow ""
+                    if (freq_names[i] != "" || (model_type == 2 && orig_model_names[j].find("MG") == string::npos)) 
+                        // empirical model also allow ""
                         model_names.push_back(orig_model_names[j] + freq_names[i]);
                 }
             } else {
@@ -543,7 +578,7 @@ int getModelList(Params &params, Alignment *aln, StrVector &models, bool separat
 //		for (i = 0; i < noptions; i++)
 //			test_options[i] = test_options_codon[i];
 //	} else 
-    if (seq_type == SEQ_MORPH || (aln->frac_const_sites == 0.0)) {
+    if (aln->frac_const_sites == 0.0) {
         // morphological or SNP data: activate +ASC
         if (with_new) {
             if (with_asc)
@@ -837,14 +872,20 @@ void testPartitionModel(Params &params, PhyloSuperTree* in_tree, vector<ModelInf
         dist[i] = -((double)this_aln->getNSeq())*this_aln->getNPattern()*this_aln->num_states;
     }
     
-    if (params.num_threads > 1)
+    if (params.num_threads > 1) 
+    {
         quicksort(dist, 0, in_tree->size()-1, distID);
-
+        if (verbose_mode >= VB_MED) {
+            for (i = 0; i < in_tree->size(); i++) {
+                cout << i+1 << "\t" << in_tree->part_info[distID[i]].name << endl;
+            }
+        }
+    }
 
 #ifdef _OPENMP
 //        for (i = 0; i < in_tree->size(); i++)
 //            cout << distID[i]+1 << "\t" << in_tree->part_info[distID[i]].name << "\t" << -dist[i] << endl;
-#pragma omp parallel for private(i) schedule(dynamic) reduction(+: lhsum, dfsum)
+#pragma omp parallel for private(i) schedule(dynamic) reduction(+: lhsum, dfsum) if(!params.model_test_and_tree)
 #endif
 	for (int j = 0; j < in_tree->size(); j++) {
         i = distID[j];
@@ -952,11 +993,11 @@ void testPartitionModel(Params &params, PhyloSuperTree* in_tree, vector<ModelInf
             this_aln = in_tree->at(distID[i] & ((1<<16)-1))->aln;
             dist[i] -= ((double)this_aln->getNSeq())*this_aln->getNPattern()*this_aln->num_states;
         }
-        if (params.num_threads > 1)
+        if (params.num_threads > 1 && num_pairs >= 1)
             quicksort(dist, 0, num_pairs-1, distID);
 
 #ifdef _OPENMP
-#pragma omp parallel for private(i) schedule(dynamic)
+#pragma omp parallel for private(i) schedule(dynamic) if(!params.model_test_and_tree)
 #endif
         for (int pair = 0; pair < num_pairs; pair++) {
             int part1 = distID[pair] >> 16;
@@ -995,6 +1036,11 @@ void testPartitionModel(Params &params, PhyloSuperTree* in_tree, vector<ModelInf
                 PhyloTree *tree = in_tree->extractSubtree(merged_set);
                 tree->setAlignment(aln);
                 extractModelInfo(set_name, model_info, part_model_info);
+//                TODO
+                tree->num_precision = in_tree->num_precision;
+                if (params.model_test_and_tree) {
+                    tree->setCheckpoint(new Checkpoint());
+                }
 //#ifdef _OPENMP
                 model = testModel(params, tree, part_model_info, this_fmodel, models_block, set_name);
 //#else
@@ -1003,6 +1049,9 @@ void testPartitionModel(Params &params, PhyloSuperTree* in_tree, vector<ModelInf
                 logl = part_model_info[0].logl;
                 df = part_model_info[0].df;
                 treelen = part_model_info[0].tree_len;
+                if (params.model_test_and_tree) {
+                    delete tree->getCheckpoint();
+                }
                 delete tree;
                 delete aln;
             }
@@ -1104,6 +1153,20 @@ void testPartitionModel(Params &params, PhyloSuperTree* in_tree, vector<ModelInf
 	in_tree->printBestPartitionRaxml((string(params.out_prefix) + ".best_scheme").c_str());
 }
 
+bool isMixtureModel(ModelsBlock *models_block, string &model_str) {
+    size_t mix_pos;
+    for (mix_pos = 0; mix_pos < model_str.length(); mix_pos++) {
+        size_t next_mix_pos = model_str.find_first_of("+*", mix_pos);
+        string sub_model_str = model_str.substr(mix_pos, next_mix_pos-mix_pos);
+        if (models_block->findMixModel(sub_model_str))
+            return true;
+        if (next_mix_pos == string::npos)
+            break;
+        mix_pos = next_mix_pos;
+    }
+    return false;
+}
+
 string testModel(Params &params, PhyloTree* in_tree, vector<ModelInfo> &model_info, ostream &fmodel, ModelsBlock *models_block,
     string set_name, bool print_mem_usage) 
 {
@@ -1143,6 +1206,8 @@ string testModel(Params &params, PhyloTree* in_tree, vector<ModelInfo> &model_in
 		// select model for each partition
 		PhyloSuperTree *stree = (PhyloSuperTree*)in_tree;
 		testPartitionModel(params, stree, model_info, fmodel, models_block);
+//        stree->linkTrees();
+        stree->mapTrees();
 		string res_models = "";
 		for (vector<PartitionInfo>::iterator it = stree->part_info.begin(); it != stree->part_info.end(); it++) {
 			if (it != stree->part_info.begin()) res_models += ",";
@@ -1154,15 +1219,26 @@ string testModel(Params &params, PhyloTree* in_tree, vector<ModelInfo> &model_in
 	in_tree->optimize_by_newton = params.optimize_by_newton;
 	in_tree->setLikelihoodKernel(params.SSE);
 
-    int num_rate_classes = 3 + params.max_rate_cats;
+//    int num_rate_classes = 3 + params.max_rate_cats;
 
-	RateHeterogeneity ** rate_class = new RateHeterogeneity*[num_rate_classes];
+	RateHeterogeneity ** rate_class = new RateHeterogeneity*[4];
 	rate_class[0] = new RateHeterogeneity();
 	rate_class[1] = new RateInvar(-1, NULL);
 	rate_class[2] = new RateGamma(params.num_rate_cats, params.gamma_shape, params.gamma_median, NULL);
-	rate_class[3] = new RateGammaInvar(params.num_rate_cats, params.gamma_shape, params.gamma_median, -1, params.optimize_model_rate_joint, NULL);
-    for (model = 4; model < num_rate_classes; model++)
-        rate_class[model] = new RateFree(model-2, params.gamma_shape, "", false, params.optimize_alg, NULL);
+	rate_class[3] = new RateGammaInvar(params.num_rate_cats, params.gamma_shape, params.gamma_median, -1, params.optimize_alg_gammai, NULL, false);
+    
+    RateFree ** rate_class_free = new RateFree*[params.max_rate_cats-1];
+    
+    for (model = 0; model < params.max_rate_cats-1; model++)
+        rate_class_free[model] = new RateFree(model+2, params.gamma_shape, "", false, params.optimize_alg, NULL);
+
+    RateFreeInvar ** rate_class_freeinvar = new RateFreeInvar*[params.max_rate_cats-1];
+    
+    for (model = 0; model < params.max_rate_cats-1; model++) {
+        rate_class_freeinvar[model] = new RateFreeInvar(model+2, params.gamma_shape, "", false, in_tree->aln->frac_const_sites/2.0, params.optimize_alg, NULL);
+        rate_class_freeinvar[model]->setFixPInvar(false);
+    }
+        
         
 	ModelGTR *subst_model = NULL;
 	if (seq_type == SEQ_BINARY)
@@ -1237,7 +1313,7 @@ string testModel(Params &params, PhyloTree* in_tree, vector<ModelInfo> &model_in
         int ncat = 0;
         string orig_name = params.model_name;
         
-        if (models_block->findMixModel(model_names[model])) {
+        if (isMixtureModel(models_block, model_names[model])) {
             // mixture model
             try {
                 mixture_model = true;
@@ -1258,7 +1334,7 @@ string testModel(Params &params, PhyloTree* in_tree, vector<ModelInfo> &model_in
             // normal model
             if (model_names[model].find("+ASC") != string::npos) {
                 model_fac->unobserved_ptns = in_tree->aln->getUnobservedConstPatterns();
-                if (model_fac->unobserved_ptns.size() == 0) {
+                if (model_fac->unobserved_ptns.size() < tree->aln->getNumNonstopCodons() || in_tree->aln->frac_const_sites > 0.0) {
                     cout.width(3);
                     cout << right << model+1 << "  ";
                     cout.width(13);
@@ -1298,7 +1374,7 @@ string testModel(Params &params, PhyloTree* in_tree, vector<ModelInfo> &model_in
             tree->setModel(subst_model);
             // initialize rate
             size_t pos;
-            if ((pos = model_names[model].find("+R")) != string::npos) {
+            if (model_names[model].find("+I") != string::npos && (pos = model_names[model].find("+R")) != string::npos) {
                 ncat = params.num_rate_cats;
                 if (model_names[model].length() > pos+2 && isdigit(model_names[model][pos+2])) {
                     ncat = convert_int(model_names[model].c_str() + pos+2);
@@ -1307,7 +1383,17 @@ string testModel(Params &params, PhyloTree* in_tree, vector<ModelInfo> &model_in
                 if (ncat <= 1) outError("Number of rate categories for " + model_names[model] + " is <= 1");
                 if (ncat > params.max_rate_cats)
                     outError("Number of rate categories for " + model_names[model] + " exceeds " + convertIntToString(params.max_rate_cats));
-                tree->setRate(rate_class[2+ncat]);
+                tree->setRate(rate_class_freeinvar[ncat-2]);
+            } else if ((pos = model_names[model].find("+R")) != string::npos) {
+                ncat = params.num_rate_cats;
+                if (model_names[model].length() > pos+2 && isdigit(model_names[model][pos+2])) {
+                    ncat = convert_int(model_names[model].c_str() + pos+2);
+    //                tree->getRate()->setNCategory(ncat);
+                }
+                if (ncat <= 1) outError("Number of rate categories for " + model_names[model] + " is <= 1");
+                if (ncat > params.max_rate_cats)
+                    outError("Number of rate categories for " + model_names[model] + " exceeds " + convertIntToString(params.max_rate_cats));
+                tree->setRate(rate_class_free[ncat-2]);
             } else if (model_names[model].find("+I") != string::npos && (pos = model_names[model].find("+G")) != string::npos) {
                 tree->setRate(rate_class[3]);
                 if (model_names[model].length() > pos+2 && isdigit(model_names[model][pos+2])) {
@@ -1350,7 +1436,8 @@ string testModel(Params &params, PhyloTree* in_tree, vector<ModelInfo> &model_in
         if (skip_model) {
             assert(prev_model_id>=0);
             size_t pos_r = info.name.find("+R");
-            if (pos_r == string::npos || info.name.substr(0, pos_r) != model_info[prev_model_id].name.substr(0, pos_r))
+            size_t prev_pos_r = model_info[prev_model_id].name.find("+R");
+            if (pos_r == string::npos || prev_pos_r == string::npos || info.name.substr(0, pos_r) != model_info[prev_model_id].name.substr(0, prev_pos_r))
                 skip_model = 0;
         }
 		for (int i = 0; i < model_info.size(); i++)
@@ -1387,6 +1474,10 @@ string testModel(Params &params, PhyloTree* in_tree, vector<ModelInfo> &model_in
                     outError("-mtree option is not supported for partition model");
                 }
                 IQTree *iqtree = new IQTree(in_tree->aln);
+                // set checkpoint
+                iqtree->setCheckpoint(in_tree->getCheckpoint());
+                iqtree->num_precision = in_tree->num_precision;
+                
                 cout << endl << "===> Testing model " << model+1 << ": " << params.model_name << endl;
                 runTreeReconstruction(params, original_model, *iqtree, model_info);
                 info.logl = iqtree->computeLikelihood();
@@ -1395,6 +1486,16 @@ string testModel(Params &params, PhyloTree* in_tree, vector<ModelInfo> &model_in
                 params.model_name = original_model;
                 params.user_file = orig_user_tree;
                 tree = iqtree;
+
+                // clear all checkpointed information
+                Checkpoint *newCheckpoint = new Checkpoint;
+                tree->getCheckpoint()->getSubCheckpoint(newCheckpoint, "iqtree");
+                tree->getCheckpoint()->clear();
+                tree->getCheckpoint()->insert(newCheckpoint->begin(), newCheckpoint->end());
+                tree->getCheckpoint()->putBool("finished", false);
+                tree->getCheckpoint()->dump(true);
+                delete newCheckpoint;
+
             } else {
                 if (tree->getMemoryRequired() > RAM_requirement) {
                     tree->deleteAllPartialLh();
@@ -1422,7 +1523,11 @@ string testModel(Params &params, PhyloTree* in_tree, vector<ModelInfo> &model_in
                     {
                         if (verbose_mode >= VB_MED)
                             cout << "reoptimizing from previous parameters of +R...." << endl;
-                        dynamic_cast<RateFree*>(rate_class[2+ncat])->setRateAndProp(dynamic_cast<RateFree*>(rate_class[1+ncat]));
+                        assert(ncat >= 3);
+                        if (tree->getRate()->getPInvar() != 0.0)                        
+                            rate_class_freeinvar[ncat-2]->setRateAndProp(rate_class_freeinvar[ncat-3]);
+                        else
+                            rate_class_free[ncat-2]->setRateAndProp(rate_class_free[ncat-3]);
                         info.logl = tree->getModelFactory()->optimizeParameters(false, false, TOL_LIKELIHOOD_MODELTEST, TOL_GRADIENT_MODELTEST);
                         info.tree_len = tree->treeLength();                        
                     }
@@ -1647,10 +1752,22 @@ string testModel(Params &params, PhyloTree* in_tree, vector<ModelInfo> &model_in
 
 	delete model_fac;
 	delete subst_model;
-	for (int rate_type = num_rate_classes-1; rate_type >= 0; rate_type--) {
+    int rate_type;
+	for (rate_type = 3; rate_type >= 0; rate_type--) {
 		delete rate_class[rate_type];
     }
     delete [] rate_class;
+    
+	for (rate_type = params.max_rate_cats-2; rate_type >= 0; rate_type--) {
+		delete rate_class_free[rate_type];
+    }
+    delete [] rate_class_free;
+
+	for (rate_type = params.max_rate_cats-2; rate_type >= 0; rate_type--) {
+		delete rate_class_freeinvar[rate_type];
+    }
+    delete [] rate_class_freeinvar;
+    
 //	delete tree_hetero;
 //	delete tree_homo;
 	in_tree->deleteAllPartialLh();
@@ -1723,6 +1840,252 @@ int countDistinctTrees(const char *filename, bool rooted, IQTree *tree, IntVecto
 
 //const double TOL_RELL_SCORE = 0.01;
 
+/*
+    Problem: solve the following linear system equation:
+    a_1*x + b_1*y = c_1
+    a_2*x + b_2*y = c_2
+    ....
+    a_n*x + b_n*y = c_n
+    
+becomes minimizing weighted least square:
+
+    sum_k { w_k*[ c_k - (a_k*x + b_k*y) ]^2 }
+
+
+the solution is:
+
+    x = [(sum_k w_k*b_k*c_k)*(sum_k w_k*a_k*b_k) - (sum_k w_k*a_k*c_k)(sum_k w_k*b_k^2)] / 
+        [ (sum_k w_k*a_k*b_k)^2 - (sum_k w_k*a_k^2)*(sum_k w_k*b_k^2) ]
+    
+    y = [(sum_k w_k*a_k*c_k)*(sum_k w_k*a_k*b_k) - (sum_k w_k*b_k*c_k)(sum_k w_k*a_k^2)] / 
+        [ (sum_k w_k*a_k*b_k)^2 - (sum_k w_k*a_k^2)*(sum_k w*k*b_k^2) ]
+    
+    @param n number of data points
+    @param w weight vector of length n
+    @param a a value vector of length n
+    @param b b value vector of length n
+    @param c c value vector of length n
+    @param[out] x x-value
+    @param[out] y y-value
+    @return least square value
+*/
+void doWeightedLeastSquare(int n, double *w, double *a, double *b, double *c, double &x, double &y) {
+    int k;
+    double BC = 0.0, AB = 0.0, AC = 0.0, A2 = 0.0, B2 = 0.0;
+    double denom;
+    for (k = 0; k < n; k++) {
+        double wa = w[k]*a[k];
+        double wb = w[k]*b[k];
+        AB += wa*b[k];
+        BC += wb*c[k];
+        AC += wa*c[k];
+        A2 += wa*a[k];
+        B2 += wb*b[k];
+    }
+    denom = 1.0/(AB*AB - A2*B2);
+    x = (BC*AB - AC*B2) * denom;
+    y = (AC*AB - BC*A2) * denom;
+}
+
+/**
+    MLE estimates for AU test
+*/
+class OptimizationAUTest : public Optimization {
+
+public:
+
+    OptimizationAUTest(double d, double c, int nscales, double *bp, double *rr, double *rr_inv) {
+        this->d = d;
+        this->c = c;
+        this->bp = bp;
+        this->rr = rr;
+        this->rr_inv = rr_inv;
+        this->nscales = nscales;
+        
+    }
+
+	/**
+		return the number of dimensions
+	*/
+	virtual int getNDim() { return 2; }
+
+
+	/**
+		the target function which needs to be optimized
+		@param x the input vector x
+		@return the function value at x
+	*/
+	virtual double targetFunk(double x[]) {
+        d = x[1];
+        c = x[2];
+        double res = 0.0;
+        for (int k = 0; k < nscales; k++) {
+            double cdf = gsl_cdf_ugaussian_P(d*rr[k] + c*rr_inv[k]);
+            res += bp[k] * log(1.0 - cdf) + (1.0-bp[k])*log(cdf);
+        }
+        return res;
+    }
+
+    void optimizeDC() {
+        double x[3], lower[3], upper[3];
+        bool bound_check[3];
+        x[1] = d;
+        x[2] = c;
+        lower[1] = lower[2] = 1e-4;
+        upper[1] = upper[2] = 100.0;
+        bound_check[1] = bound_check[2] = false;
+        minimizeMultiDimen(x, 2, lower, upper, bound_check, 1e-4);
+        d = x[1];
+        c = x[2];
+    }
+
+    double d, c;
+    int nscales;
+    double *bp;
+    double *rr;
+    double *rr_inv;
+};
+
+/**
+    @param tree_lhs RELL score matrix of size #trees x #replicates
+*/
+void performAUTest(Params &params, PhyloTree *tree, double *pattern_lhs, vector<TreeInfo> &info) {
+    
+    /* STEP 1: specify scale factors */
+    int nscales = 10;
+    double r[] = {0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 1.1, 1.2, 1.3, 1.4};
+    double rr[] = {sqrt(0.5), sqrt(0.6), sqrt(0.7), sqrt(0.8), sqrt(0.9), 1.0, 
+        sqrt(1.1), sqrt(1.2), sqrt(1.3), sqrt(1.4)};
+    double rr_inv[] = {sqrt(1/0.5), sqrt(1/0.6), sqrt(1/0.7), sqrt(1/0.8), sqrt(1/0.9), 1.0, 
+        sqrt(1/1.1), sqrt(1/1.2), sqrt(1/1.3), sqrt(1/1.4)};
+        
+    /* STEP 2: compute bootstrap proportion */
+    int ntrees = info.size();
+    size_t nboot = params.topotest_replicates;
+    double nboot_inv = 1.0 / nboot;
+    
+    int nptn = tree->getAlnNPattern();
+    int maxnptn = get_safe_upper_limit(nptn);
+    
+    double *bp = new double[ntrees*nscales];
+    memset(bp, 0, sizeof(double)*ntrees*nscales);
+    
+    int k, tid, ptn;
+#ifdef _OPENMP
+    #pragma omp parallel private(k, tid, ptn)
+    {
+    int *rstream;
+    init_random(params.ran_seed + omp_get_thread_num(), false, &rstream);
+#else
+    int *rstream = randstream;
+#endif
+    size_t boot;
+    int *boot_sample = aligned_alloc<int>(maxnptn);
+    memset(boot_sample, 0, maxnptn*sizeof(int));
+    
+    double *boot_sample_dbl = aligned_alloc<double>(maxnptn);
+    
+#ifdef _OPENMP
+    #pragma omp for schedule(static)
+#endif
+    for (k = 0; k < nscales; k++) {
+        string str = "SCALE=" + convertDoubleToString(r[k]);    
+		for (boot = 0; boot < nboot; boot++) {
+			tree->aln->createBootstrapAlignment(boot_sample, str.c_str(), rstream);
+            for (ptn = 0; ptn < maxnptn; ptn++)
+                boot_sample_dbl[ptn] = boot_sample[ptn];
+            double max_lh = -1e20;
+            int max_tid = -1;
+            for (tid = 0; tid < ntrees; tid++) {
+                double *pattern_lh = pattern_lhs + (tid*maxnptn);
+                double tree_lh;
+#ifdef BINARY32
+                tree_lh = tree->dotProductSIMD<double, Vec2d, 2>(pattern_lh, boot_sample_dbl, nptn);
+#else
+                if (instruction_set >= 7)
+                    tree_lh = tree->dotProductSIMD<double, Vec4d, 4>(pattern_lh, boot_sample_dbl, nptn);
+                else
+                    tree_lh = tree->dotProductSIMD<double, Vec2d, 2>(pattern_lh, boot_sample_dbl, nptn);
+#endif
+                if (tree_lh > max_lh) {
+                    max_lh = tree_lh;
+                    max_tid = tid;
+                } 
+            }
+            bp[k*ntrees+max_tid] += nboot_inv;
+        }
+    }
+
+    aligned_free(boot_sample_dbl);
+    aligned_free(boot_sample);
+
+#ifdef _OPENMP
+    finish_random(rstream);
+    }
+#endif
+
+    if (verbose_mode >= VB_MED) {
+        cout << "scale";
+        for (k = 0; k < nscales; k++)
+            cout << "\t" << r[k];
+        cout << endl;
+        for (tid = 0; tid < ntrees; tid++) {
+            cout << tid;
+            for (k = 0; k < nscales; k++) {
+                cout << "\t" << bp[tid+k*ntrees];
+            }
+            cout << endl;
+        }
+    }
+    
+    /* STEP 3: weighted least square fit */
+    
+    double *cc = new double[nscales];
+    double *w = new double[nscales];
+    double *this_bp = new double[nscales];
+    cout << "TreeID\tAU\tRSS\td_WLS\tc_WLS\td_MLE\tc_MLE" << endl;
+    for (tid = 0; tid < ntrees; tid++) {
+        for (k = 0; k < nscales; k++) {
+            this_bp[k] = bp[tid + k*ntrees];
+            double bp_val = min(max(bp[tid + k*ntrees], nboot_inv),1.0-nboot_inv);
+            double bp_cdf = gsl_cdf_ugaussian_Pinv(bp_val);
+            double bp_pdf = gsl_ran_ugaussian_pdf(bp_cdf);
+            cc[k] = gsl_cdf_ugaussian_Pinv(1.0 - bp_val);
+            w[k] = bp_pdf*bp_pdf*nboot / (bp_val*(1.0-bp_val));
+        }
+        double c, d; // c, d in original paper
+        // first obtain d and c by weighted least square
+        doWeightedLeastSquare(nscales, w, rr, rr_inv, cc, d, c);
+        
+        // second, perform MLE estimate of d and c
+        OptimizationAUTest mle(d, c, nscales, this_bp, rr, rr_inv);
+        mle.optimizeDC();
+        
+        // compute sum of squared difference
+        double rss = 0.0;
+        for (k = 0; k < nscales; k++) {
+            double diff = cc[k] - (rr[k]*d + rr_inv[k]*c);
+            rss += w[k] * diff * diff;
+        }
+        
+        double pchi2 = computePValueChiSquare(rss, nscales-2);
+        /* STEP 4: compute p-value according to Eq. 11 */
+        info[tid].au_pvalue = 1.0 - gsl_cdf_ugaussian_P(mle.d-mle.c);
+        cout << tid+1 << "\t" << info[tid].au_pvalue << "\t" << rss << "\t" << d << "\t" << c << "\t" << mle.d << "\t" << mle.c;
+        
+        // warning if p-value of chi-square < 0.01 (rss too high)
+        if (pchi2 < 0.01) 
+            cout << " !!!";
+        cout << endl;
+    }
+    
+    delete [] this_bp;
+    delete [] w;
+    delete [] cc;
+    delete [] bp;
+}
+
+
 void evaluateTrees(Params &params, IQTree *tree, vector<TreeInfo> &info, IntVector &distinct_ids)
 {
 	if (!params.treeset_file)
@@ -1760,18 +2123,20 @@ void evaluateTrees(Params &params, IQTree *tree, vector<TreeInfo> &info, IntVect
 		site_lh_out.close();
 	}
 
-	double time_start = getCPUTime();
+	double time_start = getRealTime();
 
 	int *boot_samples = NULL;
 	int boot;
 	//double *saved_tree_lhs = NULL;
-	double *tree_lhs = NULL;
+	double *tree_lhs = NULL; // RELL score matrix of size #trees x #replicates
 	double *pattern_lh = NULL;
 	double *pattern_lhs = NULL;
-	double *orig_tree_lh = NULL;
+	double *orig_tree_lh = NULL; // Original tree log-likelihoods
 	double *max_lh = NULL;
 	double *lhdiff_weights = NULL;
 	int nptn = tree->getAlnNPattern();
+    int maxnptn = get_safe_upper_limit(nptn);
+    
 	if (params.topotest_replicates && ntrees > 1) {
 		size_t mem_size = (size_t)params.topotest_replicates*nptn*sizeof(int) +
 				ntrees*params.topotest_replicates*sizeof(double) +
@@ -1790,14 +2155,16 @@ void evaluateTrees(Params &params, IQTree *tree, vector<TreeInfo> &info, IntVect
 		//	outError(ERR_NO_MEMORY);
 		if (!(tree_lhs = new double [ntrees * params.topotest_replicates]))
 			outError(ERR_NO_MEMORY);
-		if (params.do_weighted_test) {
+		if (params.do_weighted_test || params.do_au_test) {
 			if (!(lhdiff_weights = new double [ntrees * ntrees]))
 				outError(ERR_NO_MEMORY);
-			if (!(pattern_lhs = new double[ntrees* nptn]))
-				outError(ERR_NO_MEMORY);
+            pattern_lhs = aligned_alloc<double>(ntrees*maxnptn);
+//			if (!(pattern_lhs = new double[ntrees* nptn]))
+//				outError(ERR_NO_MEMORY);
 		}
-		if (!(pattern_lh = new double[nptn]))
-			outError(ERR_NO_MEMORY);
+        pattern_lh = aligned_alloc<double>(maxnptn);
+//		if (!(pattern_lh = new double[nptn]))
+//			outError(ERR_NO_MEMORY);
 		if (!(orig_tree_lh = new double[ntrees]))
 			outError(ERR_NO_MEMORY);
 		if (!(max_lh = new double[params.topotest_replicates]))
@@ -1824,13 +2191,13 @@ void evaluateTrees(Params &params, IQTree *tree, vector<TreeInfo> &info, IntVect
         tree->setRootNode(params.root);
 		if (tree->isSuperTree())
 			((PhyloSuperTree*) tree)->mapTrees();
-		if ((tree->sse == LK_EIGEN || tree->sse == LK_EIGEN_SSE) && !tree->isBifurcating()) {
-			cout << "NOTE: Changing to old kernel as user tree is multifurcating" << endl;
-			if (tree->sse == LK_EIGEN)
-				tree->changeLikelihoodKernel(LK_NORMAL);
-			else
-				tree->changeLikelihoodKernel(LK_SSE);
-		}
+//		if ((tree->sse == LK_EIGEN || tree->sse == LK_EIGEN_SSE) && !tree->isBifurcating()) {
+//			cout << "NOTE: Changing to old kernel as user tree is multifurcating" << endl;
+//			if (tree->sse == LK_EIGEN)
+//				tree->changeLikelihoodKernel(LK_NORMAL);
+//			else
+//				tree->changeLikelihoodKernel(LK_SSE);
+//		}
 
 		tree->initializeAllPartialLh();
 		tree->fixNegativeBranch(false);
@@ -1849,9 +2216,10 @@ void evaluateTrees(Params &params, IQTree *tree, vector<TreeInfo> &info, IntVect
 
 		if (pattern_lh) {
 			double curScore = tree->getCurScore();
+            memset(pattern_lh, 0, maxnptn*sizeof(double));
 			tree->computePatternLikelihood(pattern_lh, &curScore);
-			if (params.do_weighted_test)
-				memcpy(pattern_lhs + tid*nptn, pattern_lh, nptn*sizeof(double));
+			if (params.do_weighted_test || params.do_au_test)
+				memcpy(pattern_lhs + tid*maxnptn, pattern_lh, maxnptn*sizeof(double));
 		}
 		if (params.print_site_lh) {
 			string tree_name = "Tree" + convertIntToString(tree_index+1);
@@ -1884,7 +2252,7 @@ void evaluateTrees(Params &params, IQTree *tree, vector<TreeInfo> &info, IntVect
 		int *tree_ranks = new int[ntrees];
 
 		/* perform RELL BP method */
-		cout << "Performing RELL test..." << endl;
+		cout << "Performing RELL-BP test..." << endl;
 		int *maxtid = new int[params.topotest_replicates];
 		double *maxL = new double[params.topotest_replicates];
 		int *maxcount = new int[params.topotest_replicates];
@@ -1993,10 +2361,10 @@ void evaluateTrees(Params &params, IQTree *tree, vector<TreeInfo> &info, IntVect
 			cout << "Computing pairwise logl difference variance ..." << endl;
 			/* computing lhdiff_weights as 1/sqrt(lhdiff_variance) */
 			for (tid = 0; tid < ntrees; tid++) {
-				double *pattern_lh1 = pattern_lhs + (tid * nptn);
+				double *pattern_lh1 = pattern_lhs + (tid * maxnptn);
 				lhdiff_weights[tid*ntrees+tid] = 0.0;
 				for (tid2 = tid+1; tid2 < ntrees; tid2++) {
-					double lhdiff_variance = tree->computeLogLDiffVariance(pattern_lh1, pattern_lhs + (tid2*nptn));
+					double lhdiff_variance = tree->computeLogLDiffVariance(pattern_lh1, pattern_lhs + (tid2*maxnptn));
 					lhdiff_weights[tid*ntrees+tid2] = 1.0/sqrt(lhdiff_variance);
 					lhdiff_weights[tid2*ntrees+tid] = lhdiff_weights[tid*ntrees+tid2];
 				}
@@ -2036,6 +2404,9 @@ void evaluateTrees(Params &params, IQTree *tree, vector<TreeInfo> &info, IntVect
 				info[tid].wkh_pvalue /= params.topotest_replicates;
 			}
 		}
+        
+        delete [] avg_lh;
+        
 		/* now to ELW - Expected Likelihood Weight method */
 		cout << "Performing ELW test..." << endl;
 
@@ -2082,6 +2453,11 @@ void evaluateTrees(Params &params, IQTree *tree, vector<TreeInfo> &info, IntVect
 			outError("Internal error: Wrong ", __func__);
 		delete [] sumL;
 
+        if (params.do_au_test) {
+            cout << "Performing approximately unbiased (AU) test..." << endl;
+            performAUTest(params, tree, pattern_lhs, info);
+        }
+
 		delete [] tree_ranks;
 		delete [] tree_probs;
 
@@ -2091,9 +2467,9 @@ void evaluateTrees(Params &params, IQTree *tree, vector<TreeInfo> &info, IntVect
 	if (orig_tree_lh)
 		delete [] orig_tree_lh;
 	if (pattern_lh)
-		delete [] pattern_lh;
+        aligned_free(pattern_lh);
 	if (pattern_lhs)
-		delete [] pattern_lhs;
+        aligned_free(pattern_lhs);
 	if (lhdiff_weights)
 		delete [] lhdiff_weights;
 	if (tree_lhs)
@@ -2110,7 +2486,7 @@ void evaluateTrees(Params &params, IQTree *tree, vector<TreeInfo> &info, IntVect
 	treeout.close();
 	in.close();
 
-	cout << "Time for evaluating all trees: " << getCPUTime() - time_start << " sec." << endl;
+	cout << "Time for evaluating all trees: " << getRealTime() - time_start << " sec." << endl;
 
 }
 
@@ -2120,3 +2496,6 @@ void evaluateTrees(Params &params, IQTree *tree) {
 	IntVector distinct_ids;
 	evaluateTrees(params, tree, info, distinct_ids);
 }
+
+
+
