@@ -101,10 +101,11 @@ void SuperAlignment::linkSubAlignment(int part) {
 	}
 }
 
-void SuperAlignment::extractSubAlignment(Alignment *aln, IntVector &seq_id, int min_true_char) {
+void SuperAlignment::extractSubAlignment(Alignment *aln, IntVector &seq_id, int min_true_char, int min_taxa, IntVector *kept_partitions) {
 	assert(aln->isSuperAlignment());
 	SuperAlignment *saln = (SuperAlignment*)aln;
 
+    int i;
     IntVector::iterator it;
     for (it = seq_id.begin(); it != seq_id.end(); it++) {
         assert(*it >= 0 && *it < aln->getNSeq());
@@ -115,24 +116,33 @@ void SuperAlignment::extractSubAlignment(Alignment *aln, IntVector &seq_id, int
 	//Alignment::extractSubAlignment(aln, seq_id, 0);
 
 	taxa_index.resize(getNSeq());
-	for (int i = 0; i < getNSeq(); i++)
+	for (i = 0; i < getNSeq(); i++)
 		taxa_index[i].resize(saln->partitions.size(), -1);
 
 	int part = 0;
-	partitions.resize(saln->partitions.size());
+//	partitions.resize(saln->partitions.size());
+    partitions.resize(0);
 	for (vector<Alignment*>::iterator ait = saln->partitions.begin(); ait != saln->partitions.end(); ait++, part++) {
 		IntVector sub_seq_id;
 		for (IntVector::iterator it = seq_id.begin(); it != seq_id.end(); it++)
 			if (saln->taxa_index[*it][part] >= 0)
 				sub_seq_id.push_back(saln->taxa_index[*it][part]);
+        if (sub_seq_id.size() < min_taxa)
+            continue;
 		Alignment *subaln = new Alignment;
 		subaln->extractSubAlignment(*ait, sub_seq_id, 0);
-		partitions[part] = subaln;
-		linkSubAlignment(part);
+		partitions.push_back(subaln);
+		linkSubAlignment(partitions.size()-1);
+        if (kept_partitions) kept_partitions->push_back(part);
 //		cout << subaln->getNSeq() << endl;
 //		subaln->printPhylip(cout);
 	}
 
+    if (partitions.size() < saln->partitions.size()) {
+        for (i = 0; i < getNSeq(); i++)
+            taxa_index[i].resize(partitions.size());
+    }
+
 	// now build the patterns based on taxa_index
 	buildPattern();
 }
@@ -319,9 +329,9 @@ void SuperAlignment::createBootstrapAlignment(IntVector &pattern_freq, const cha
 }
 
 
-void SuperAlignment::createBootstrapAlignment(int *pattern_freq, const char *spec) {
+void SuperAlignment::createBootstrapAlignment(int *pattern_freq, const char *spec, int *rstream) {
 	if (!isSuperAlignment()) outError("Internal error: ", __func__);
-	if (spec && strncmp(spec, "GENE", 4) != 0) outError("Unsupported yet. ", __func__);
+//	if (spec && strncmp(spec, "GENE", 4) != 0) outError("Unsupported yet. ", __func__);
 
 	if (spec && strncmp(spec, "GENE", 4) == 0) {
 		// resampling whole genes
@@ -333,12 +343,12 @@ void SuperAlignment::createBootstrapAlignment(int *pattern_freq, const char *spe
 		}
 		memset(pattern_freq, 0, nptn * sizeof(int));
 		for (int i = 0; i < partitions.size(); i++) {
-			int part = random_int(partitions.size());
+			int part = random_int(partitions.size(), rstream);
 			Alignment *aln = partitions[part];
 			if (strncmp(spec,"GENESITE",8) == 0) {
 				// then resampling sites in resampled gene
 				for (int j = 0; j < aln->getNSite(); j++) {
-					int ptn_id = aln->getPatternID(random_int(aln->getNPattern()));
+					int ptn_id = aln->getPatternID(random_int(aln->getNPattern(), rstream));
 					pattern_freq[ptn_id + part_pos[part]]++;
 				}
 
@@ -351,7 +361,10 @@ void SuperAlignment::createBootstrapAlignment(int *pattern_freq, const char *spe
 		// resampling sites within genes
 		int offset = 0;
 		for (vector<Alignment*>::iterator it = partitions.begin(); it != partitions.end(); it++) {
-			(*it)->createBootstrapAlignment(pattern_freq + offset);
+            if (spec && strncmp(spec, "SCALE=", 6) == 0)
+                (*it)->createBootstrapAlignment(pattern_freq + offset, spec, rstream);
+            else
+                (*it)->createBootstrapAlignment(pattern_freq + offset, NULL, rstream);
 			offset += (*it)->getNPattern();
 		}
 	}
@@ -429,20 +442,26 @@ SuperAlignment::~SuperAlignment()
 	partitions.clear();
 }
 
-void SuperAlignment::printCombinedAlignment(ostream &out, bool append) {
+void SuperAlignment::printCombinedAlignment(ostream &out, bool print_taxid) {
 	vector<Alignment*>::iterator pit;
 	int final_length = 0;
 	for (pit = partitions.begin(); pit != partitions.end(); pit++)
-		final_length += (*pit)->getNSite();
+        if ((*pit)->seq_type == SEQ_CODON)
+            final_length += 3*(*pit)->getNSite();
+        else
+            final_length += (*pit)->getNSite();
 
 	out << getNSeq() << " " << final_length << endl;
-	StrVector::iterator it;
 	int max_len = getMaxSeqNameLength();
+    if (print_taxid) max_len = 10;
 	if (max_len < 10) max_len = 10;
-	int seq_id = 0;
-	for (it = seq_names.begin(); it != seq_names.end(); it++, seq_id++) {
+	int seq_id;
+	for (seq_id = 0; seq_id < seq_names.size(); seq_id++) {
 		out.width(max_len);
-		out << left << (*it) << " ";
+        if (print_taxid)
+            out << left << seq_id << " ";
+        else
+            out << left << seq_names[seq_id] << " ";
 		int part = 0;
 		for (pit = partitions.begin(); pit != partitions.end(); pit++, part++) {
 			int part_seq_id = taxa_index[seq_id][part];
@@ -460,13 +479,6 @@ void SuperAlignment::printCombinedAlignment(ostream &out, bool append) {
 }
 
 void SuperAlignment::printCombinedAlignment(const char *file_name, bool append) {
-	vector<Alignment*>::iterator pit;
-	int final_length = 0;
-	for (pit = partitions.begin(); pit != partitions.end(); pit++)
-        if ((*pit)->seq_type == SEQ_CODON)
-            final_length += 3*(*pit)->getNSite();
-        else
-            final_length += (*pit)->getNSite();
 	try {
 		ofstream out;
 		out.exceptions(ios::failbit | ios::badbit);
@@ -475,28 +487,7 @@ void SuperAlignment::printCombinedAlignment(const char *file_name, bool append)
 			out.open(file_name, ios_base::out | ios_base::app);
 		else
 			out.open(file_name);
-		out << getNSeq() << " " << final_length << endl;
-		StrVector::iterator it;
-		int max_len = getMaxSeqNameLength();
-		if (max_len < 10) max_len = 10;
-		int seq_id = 0;
-		for (it = seq_names.begin(); it != seq_names.end(); it++, seq_id++) {
-			out.width(max_len);
-			out << left << (*it) << " ";
-			int part = 0;
-			for (pit = partitions.begin(); pit != partitions.end(); pit++, part++) {
-				int part_seq_id = taxa_index[seq_id][part];
-				int nsite = (*pit)->getNSite();
-				if (part_seq_id >= 0) {
-					for (int i = 0; i < nsite; i++)
-						out << (*pit)->convertStateBackStr((*pit)->getPattern(i) [part_seq_id]);
-				} else {
-					string str(nsite, '?');
-					out << str;
-				}
-			}
-			out << endl;
-		}
+        printCombinedAlignment(out);
 		out.close();
 		cout << "Concatenated alignment was printed to " << file_name << endl;
 	} catch (ios::failure) {
